import numpy as np
import math

from math import floor, pi, log, tan, atan, exp

import urllib.request as ur
import PIL.Image as pil
import io


import multiprocessing
import time
from tqdm import tqdm
import os
import sys


from osgeo import gdal
from osgeo import ogr
from osgeo import osr
from osgeo import gdal_array
from osgeo import gdalconst

# WGS-84 to Web Mercator

def wgs_to_mercator(x, y):
    y = 85.0511287798 if y > 85.0511287798 else y
    y = -85.0511287798 if y < -85.0511287798 else y
    x2 = x * 20037508.34 / 180
    y2 = log(tan((90 + y) * pi / 360)) / (pi / 180)
    y2 = y2 * 20037508.34 / 180
    return x2, y2

# Web Mercator to WGS-84

def mercator_to_wgs(x, y):
    x2 = x / 20037508.34 * 180
    y2 = y / 20037508.34 * 180
    y2 = 180 / pi * (2 * atan(exp(y2 * pi / 180)) - pi / 2)
    return x2, y2
# --------------------------------------------------------------------------------------

# -----------------Interchange between GCJ-02 to WGS-84-------------------
# All public geographic data in mainland China need to be encrypted with GCJ-02, introducing random bias
# This part of the code is used to remove the bias


def transformLat(x, y):
    ret = -100.0 + 2.0 * x + 3.0 * y + 0.2 * y * \
        y + 0.1 * x * y + 0.2 * math.sqrt(abs(x))
    ret += (20.0 * math.sin(6.0 * x * math.pi) + 20.0 *
            math.sin(2.0 * x * math.pi)) * 2.0 / 3.0
    ret += (20.0 * math.sin(y * math.pi) + 40.0 *
            math.sin(y / 3.0 * math.pi)) * 2.0 / 3.0
    ret += (160.0 * math.sin(y / 12.0 * math.pi) + 320 *
            math.sin(y * math.pi / 30.0)) * 2.0 / 3.0
    return ret


def transformLon(x, y):
    ret = 300.0 + x + 2.0 * y + 0.1 * x * x + \
        0.1 * x * y + 0.1 * math.sqrt(abs(x))
    ret += (20.0 * math.sin(6.0 * x * math.pi) + 20.0 *
            math.sin(2.0 * x * math.pi)) * 2.0 / 3.0
    ret += (20.0 * math.sin(x * math.pi) + 40.0 *
            math.sin(x / 3.0 * math.pi)) * 2.0 / 3.0
    ret += (150.0 * math.sin(x / 12.0 * math.pi) + 300.0 *
            math.sin(x / 30.0 * math.pi)) * 2.0 / 3.0
    return ret


def delta(lat, lon):
    '''
    Krasovsky 1940
    //
    // a = 6378245.0, 1/f = 298.3
    // b = a * (1 - f)
    // ee = (a^2 - b^2) / a^2;
    '''
    a = 6378245.0  # a: Projection factor of satellite ellipsoidal coordinates projected onto a flat map coordinate system
    ee = 0.00669342162296594323  # ee: Eccentricity of ellipsoid
    dLat = transformLat(lon - 105.0, lat - 35.0)
    dLon = transformLon(lon - 105.0, lat - 35.0)
    radLat = lat / 180.0 * math.pi
    magic = math.sin(radLat)
    magic = 1 - ee * magic * magic
    sqrtMagic = math.sqrt(magic)
    dLat = (dLat * 180.0) / ((a * (1 - ee)) / (magic * sqrtMagic) * math.pi)
    dLon = (dLon * 180.0) / (a / sqrtMagic * math.cos(radLat) * math.pi)
    return {'lat': dLat, 'lon': dLon}


def outOfChina(lat, lon):
    if (lon < 72.004 or lon > 137.8347):
        return True
    if (lat < 0.8293 or lat > 55.8271):
        return True
    return False


def gcj_to_wgs(gcjLon, gcjLat):
    if outOfChina(gcjLat, gcjLon):
        return (gcjLon, gcjLat)
    d = delta(gcjLat, gcjLon)
    return (gcjLon - d["lon"], gcjLat - d["lat"])


def wgs_to_gcj(wgsLon, wgsLat):
    if outOfChina(wgsLat, wgsLon):
        return wgsLon, wgsLat
    d = delta(wgsLat, wgsLon)
    return wgsLon + d["lon"], wgsLat + d["lat"]
# --------------------------------------------------------------

# ---------------------------------------------------------
# Get tile coordinates in Google Maps based on latitude and longitude of WGS-84


def wgs_to_tile(j, w, z):
    '''
    Get google-style tile cooridinate from geographical coordinate
    j : Longittude
    w : Latitude
    z : zoom
    '''
    def isnum(x): return isinstance(x, int) or isinstance(x, float)
    if not(isnum(j) and isnum(w)):
        raise TypeError("j and w must be int or float!")

    if not isinstance(z, int) or z < 0 or z > 22:
        raise TypeError("z must be int and between 0 to 22.")

    if j < 0:
        j = 180 + j
    else:
        j += 180
    j /= 360  # make j to (0,1)

    w = 85.0511287798 if w > 85.0511287798 else w
    w = -85.0511287798 if w < -85.0511287798 else w
    w = log(tan((90 + w) * pi / 360)) / (pi / 180)
    w /= 180  # make w to (-1,1)
    w = 1 - (w + 1) / 2  # make w to (0,1) and left top is 0-point

    num = 2**z
    x = floor(j * num)
    y = floor(w * num)
    return x, y


def pixls_to_mercator(zb):
    # Get the web Mercator projection coordinates of the four corners of the
    # area according to the four corner coordinates of the tile
    inx, iny = zb["LT"]  # left top
    inx2, iny2 = zb["RB"]  # right bottom
    length = 20037508.3427892
    sum = 2**zb["z"]
    LTx = inx / sum * length * 2 - length
    LTy = -(iny / sum * length * 2) + length

    RBx = (inx2 + 1) / sum * length * 2 - length
    RBy = -((iny2 + 1) / sum * length * 2) + length

    # LT=left top,RB=right buttom
    # Returns the projected coordinates of the four corners
    res = {'LT': (LTx, LTy), 'RB': (RBx, RBy),
           'LB': (LTx, RBy), 'RT': (RBx, LTy)}
    return res


def tile_to_pixls(zb):
    # Tile coordinates are converted to pixel coordinates of the four corners
    out = {}
    width = (zb["RT"][0] - zb["LT"][0] + 1) * Tilesize
    height = (zb["LB"][1] - zb["LT"][1] + 1) * Tilesize
    out["LT"] = (0, 0)
    out["RT"] = (width, 0)
    out["LB"] = (0, -height)
    out["RB"] = (width, -height)
    return out



def getExtent(x1, y1, x2, y2, z, source="Google China"):
    pos1x, pos1y = wgs_to_tile(x1, y1, z)
    pos2x, pos2y = wgs_to_tile(x2, y2, z)
    Xframe = pixls_to_mercator({"LT": (pos1x, pos1y), "RT": (
        pos2x, pos1y), "LB": (pos1x, pos2y), "RB": (pos2x, pos2y), "z": z})
    for i in ["LT", "LB", "RT", "RB"]:
        Xframe[i] = mercator_to_wgs(*Xframe[i])
    if source == "Google":
        pass
    elif source == "Google China":
        for i in ["LT", "LB", "RT", "RB"]:
            Xframe[i] = gcj_to_wgs(*Xframe[i])
    else:
        pass
        # raise Exception("Invalid argument: source.")
    return Xframe


def saveTiff(r, g, b, gt, filePath):
    # print("save:",gt)
    if type(r)==pil.Image:
        r,g,b=np.array(r),np.array(g),np.array(b)
    driver = gdal.GetDriverByName('GTiff')
    # Create a 3-band dataset
    outRaster = driver.Create(filePath,
        r.shape[1],
        r.shape[0],
        3,
        gdal.GDT_Byte)
    outRaster.SetGeoTransform(gt)
    try:
        outRasterSRS = osr.SpatialReference()
        outRasterSRS.ImportFromEPSG(4326)
        outRaster.SetProjection(outRasterSRS.ExportToWkt())
    except Exception as e:
        print(e)
    outRaster.GetRasterBand(1).WriteArray(r)
    outRaster.GetRasterBand(2).WriteArray(g)
    outRaster.GetRasterBand(3).WriteArray(b)
    outRaster.FlushCache()
    outRaster = None
    # print("Image Saved")
# ---------------------------------------------------------





def TileXYToQuadKey(tileX, tileY, level):
    quadKey = ""
    for i in range(-level, 0):
        digit = 0
        mask = 1 << (-i - 1)
        if (tileX & mask) != 0:
            digit += 1
        if (tileY & mask) != 0:
            digit += 1
            digit += 1
        quadKey += str(digit)
    return quadKey


def get_url(source, x, y, z, style):
    if source == 'Google China':
        url = MAP_URLS["Google China"].format(x=x, y=y, z=z, style=style)
    elif source == "Bing VirtualEarth":
        quadkey = TileXYToQuadKey(x, y, z)
        url = MAP_URLS["Bing VirtualEarth"].format(q=quadkey)

    elif source == 'Google':
        url = MAP_URLS["Google"].format(x=x, y=y, z=z, style=style)
    else:
        url = MAP_URLS[source].format(x=x, y=y, z=z)
        # pass
        # raise Exception("Unknown Map Source ! ")
    return url


def get_urls(x1, y1, x2, y2, z, source, style):
    pos1x, pos1y = wgs_to_tile(x1, y1, z)
    pos2x, pos2y = wgs_to_tile(x2, y2, z)
    lenx = pos2x - pos1x + 1
    leny = pos2y - pos1y + 1
    print("-----Total tiles number：{x} X {y}".format(x=lenx, y=leny))
    urls = [
        get_url(
            source,
            i,
            j,
            z,
            style) for j in range(pos1y,pos1y +leny) for i in range(pos1x,pos1x +lenx)
            ]
    return urls
# ---------------------------------------------------------

# ---------------------------------------------------------



def merge_tiles(datas, x1, y1, x2, y2, z):
    pos1x, pos1y = wgs_to_tile(x1, y1, z)
    pos2x, pos2y = wgs_to_tile(x2, y2, z)
    lenx = pos2x - pos1x + 1
    leny = pos2y - pos1y + 1
    outpic = pil.new('RGBA', (lenx * Tilesize, leny * Tilesize))
    for i, data in enumerate(datas):
        picio = io.BytesIO(data)
        small_pic = pil.open(picio)
        y, x = i // lenx, i % lenx
        outpic.paste(small_pic, (x * Tilesize, y * Tilesize))
    print('-----Tiles merge completed')
    return outpic



def download_tiles(url):
    data=[None]
    task=urloader(url,data)
    task.start()
    task.join()
    print(multiprocessing.current_process())
    return task.datas


# ---------------------------------------------------------

# ---------------------------------------------------------

from threading import Thread

class urloader(Thread):
    # multiple threads downloader
    def __init__(self, urls, datas):
        # index represents the number of threads
        # count represents the total number of threads
        # urls represents the list of URLs nedd to be downloaded
        # datas represents the list of data need to be returned.
        super().__init__()
        self.urls = urls
        self.datas = datas

    def download(self, url):
        HEADERS = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.76 Safari/537.36'
            }
        header = ur.Request(url, headers=HEADERS)
        err = 0
        while(err < 3):
            try:
                data = ur.urlopen(header).read()
            except BaseException:
                err += 1
            else:
                return data
        raise Exception("Bad network link.")

    def run(self):
        url=self.urls
        self.datas = self.download(url)
        print(self.datas)



MAP_URLS = {
    "Google": "http://mts0.googleapis.com/vt?lyrs={style}&x={x}&y={y}&z={z}",
    "Google China": "http://mt2.google.cn/vt/lyrs={style}&hl=zh-CN&gl=CN&src=app&x={x}&y={y}&z={z}",

    "Google Maps": "https://mt1.google.com/vt/lyrs=m&x={x}&y={y}&z={z}",
    "Google Satellite": "https://mt1.google.com/vt/lyrs=s&x={x}&y={y}&z={z}",
    "Google Terrain": "https://mt1.google.com/vt/lyrs=t&x={x}&y={y}&z={z}",
    "Google Terrain Hybrid": "https://mt1.google.com/vt/lyrs=p&x={x}&y={y}&z={z}",
    "Google Satellite Hybrid": "https://mt1.google.com/vt/lyrs=y&x={x}&y={y}&z={z}",

    "Stamen Terrain": "http://tile.stamen.com/terrain/{z}/{x}/{y}.png",
    "Stamen Toner": "http://tile.stamen.com/toner/{z}/{x}/{y}.png",
    "Stamen Toner Light": "http://tile.stamen.com/toner-lite/{z}/{x}/{y}.png",
    "Stamen Watercolor": "http://tile.stamen.com/watercolor/{z}/{x}/{y}.jpg",

    "Wikimedia Map": "https://maps.wikimedia.org/osm-intl/{z}/{x}/{y}.png",
    "Wikimedia Hike Bike Map": "http://tiles.wmflabs.org/hikebike/{z}/{x}/{y}.png",

    "Esri Boundaries Places": "https://server.arcgisonline.com/ArcGIS/rest/services/Reference/World_Boundaries_and_Places/MapServer/tile/{z}/{y}/{x}",
    "Esri Gray (dark)": "http://services.arcgisonline.com/ArcGIS/rest/services/Canvas/World_Dark_Gray_Base/MapServer/tile/{z}/{y}/{x}",
    "Esri Gray (light)": "http://services.arcgisonline.com/ArcGIS/rest/services/Canvas/World_Light_Gray_Base/MapServer/tile/{z}/{y}/{x}",
    "Esri National Geographic": "http://services.arcgisonline.com/ArcGIS/rest/services/NatGeo_World_Map/MapServer/tile/{z}/{y}/{x}",
    "Esri Ocean": "https://services.arcgisonline.com/ArcGIS/rest/services/Ocean/World_Ocean_Base/MapServer/tile/{z}/{y}/{x}",
    "Esri Satellite": "https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}",
    "Esri Standard": "https://server.arcgisonline.com/ArcGIS/rest/services/World_Street_Map/MapServer/tile/{z}/{y}/{x}",
    "Esri Terrain": "https://server.arcgisonline.com/ArcGIS/rest/services/World_Terrain_Base/MapServer/tile/{z}/{y}/{x}",
    "Esri Transportation": "https://server.arcgisonline.com/ArcGIS/rest/services/Reference/World_Transportation/MapServer/tile/{z}/{y}/{x}",
    "Esri Topo World": "http://services.arcgisonline.com/ArcGIS/rest/services/World_Topo_Map/MapServer/tile/{z}/{y}/{x}",

    "OpenStreetMap Standard": "http://tile.openstreetmap.org/{z}/{x}/{y}.png",
    "OpenStreetMap H.O.T.": "http://tile.openstreetmap.fr/hot/{z}/{x}/{y}.png",
    "OpenStreetMap Monochrome": "http://tiles.wmflabs.org/bw-mapnik/{z}/{x}/{y}.png",
    "OpenTopoMap": "https://tile.opentopomap.org/{z}/{x}/{y}.png",

    "Strava All": "https://heatmap-external-b.strava.com/tiles/all/bluered/{z}/{x}/{y}.png",
    "Strava Run": "https://heatmap-external-b.strava.com/tiles/run/bluered/{z}/{x}/{y}.png?v=19",

    "Open Weather Map Temperature": "http://tile.openweathermap.org/map/temp_new/{z}/{x}/{y}.png?APPID=1c3e4ef8e25596946ee1f3846b53218a",
    "Open Weather Map Clouds": "http://tile.openweathermap.org/map/clouds_new/{z}/{x}/{y}.png?APPID=ef3c5137f6c31db50c4c6f1ce4e7e9dd",
    "Open Weather Map Wind Speed": "http://tile.openweathermap.org/map/wind_new/{z}/{x}/{y}.png?APPID=f9d0069aa69438d52276ae25c1ee9893",

    "CartoDb Dark Matter": "http://basemaps.cartocdn.com/dark_all/{z}/{x}/{y}.png",
    "CartoDb Positron": "http://basemaps.cartocdn.com/light_all/{z}/{x}/{y}.png",

    "Bing VirtualEarth": "http://ecn.t3.tiles.virtualearth.net/tiles/a{q}.jpeg?g=1"

}

Tilesize = 256


AGENT="Mozilla/5.0 (Macintosh; Intel Mac OS X 10_16_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.83 Safari/537.36"



import multiprocessing
from tqdm import tqdm
import os



from .downloader import downloader
from .obsclient import bucket
from .vector import Vector

from osgeo import gdal
from osgeo import ogr
from osgeo import osr
from osgeo import gdal_array
from osgeo import gdalconst

def Process(
    VectorDataSource,
    WgsCord,
    Class_key,
    DataSourcesType='Google China',
    DataSetName="DataSet",
    Remote_dataset_root="DataSets/",
    Thread_count=2,
    Nodata=0,
    Merge=False,
    Keep_local=True,
    Over_write=True,
    Upload=False,
    **args
    ):
    """
    Step I:

        Init Downlaoder,Bucket,Vector

    Step II:

        Init default vector layer
        Init area , imagery level of mission

    Step III:

        Download
        Merge(Optional)
        Rasterize

    Step IV:

        Upload to Bucket

    Last Step:

        If don't save temp dataset ,clean the cache
    args:

        for obs server:
            ak : access_key_id,    
            sk : secret_access_key,    
            server : server
            bn :  bucketname


    """

    print("\033[1;32# ---------------------------------------------------------------------------- #\033[0m")
    print("\033[1;32#                                     DARTH                                    #\033[0m")
    print("\033[1;32# ---------------------------------------------------------------------------- #\033[0m")

    print("# ===== Bucket para preview\033[1;32 %s\033[0m"%args)


    
    print("\n\n\n# ---------------------------------------------------------------------------- #")
    print("# ---------------------------------- Step I ---------------------------------- #")
    print("# ---------------------------------------------------------------------------- #")



    Download=downloader(DataSourcesType,thread_count=Thread_count)
    if Upload:
        Bucket=bucket(
            access_key_id=args["ak"],
            secret_access_key=args["sk"],
            server=args["server"],
            bucketName=args["bn"]
            )
        if not Over_write:
            Bucket.check(remote_metaname)
    Vec=Vector(VectorDataSource)
    remote_metaname=Remote_dataset_root+DataSetName+"/.meta"





    print("\n\n\n# ---------------------------------------------------------------------------- #")
    print("# ---------------------------------- Step II --------------------------------- #")
    print("# ---------------------------------------------------------------------------- #")

    Vec.getDefaultLayerbyName(Class_key)
    Download.add_cord(*WgsCord)
    Vec.crop_default_layer_by_rect(Download.mercator_cord)

    print("\n\n\n# ---------------------------------------------------------------------------- #")
    print("# --------------------------------- Step III --------------------------------- #")
    print("# ---------------------------------------------------------------------------- #")

    image_dir=os.path.join(DataSetName,'images/')
    targets_dir=os.path.join(DataSetName,'targets/')
    print("# ===== imagery dir :\033[1;32%s\033[0m"%image_dir)
    print("# ===== targets dir :\033[1;32%s\033[0m"%targets_dir)
    if not os.path.exists("./"+DataSetName):
        os.makedirs(image_dir)
        os.makedirs(targets_dir)
    

    local_metaname=DataSetName+"/.meta"
    with open(local_metaname,"w") as meta:
        if Upload:
            meta.write(
                "Bucket Meta:\n"+str(Bucket.getBucketMetadata())
            )
        meta.write(
            "Vector object Meta:\n"+str(Vec.meta)
        )
    meta.close()

        
    if Upload:
        bucket_imagery_root=os.path.join(Remote_dataset_root,image_dir)
        bucket_targets_root=os.path.join(Remote_dataset_root,targets_dir)
        bucket_description_root=os.path.join(Remote_dataset_root,DataSetName+"/")
            
        print("# ===== Bucket imagery root  :\033[1;32%s\033[0m",bucket_imagery_root)
        print("# ===== Bucket Targets root  :\033[1;32%s\033[0m",bucket_targets_root)
        print("# ===== Bucket Description root :\033[1;32%s\033[0m",bucket_description_root)
        
        Bucket.cd("DataSets")
        Bucket.ls()
    
    print("\033[5;36# ===== Start Downloading.....\033[0m")
    Download.download(output_path=image_dir)
    tiles=[i["path"] for i in Download.result]
    Vec.generate(tiles,output_path=targets_dir)
    


    if Upload:
        print("\n\n\n# ---------------------------------------------------------------------------- #")
        print("# ---------------------------------- Step IV --------------------------------- #")
        print("# ---------------------------------------------------------------------------- #")
        print("# ===== Upload dataset meta\033[1;32%s\033[0m"%remote_metaname)

        Bucket.upload(
            remote_path=remote_metaname,
            local_path=local_metaname
        )
            
        ## Saveing index json file
        remote_json_path=os.path.join(bucket_description_root,Download.json_path.split('/')[-1])
        print("# ===== Upload dataset description\033[1;32%s\033[0m"%remote_json_path)
        if not Over_write:
            Bucket.check(remote_json_path)
        
        Bucket.upload(
                remote_path=remote_json_path,
                local_path=Download.json_path
            )


        print("# ===== upload imagry to bucket.....")
        
        for tile in tqdm(tiles):
            file_name=tile.split('/')[-1]
            remote_tiles=os.path.join(bucket_imagery_root,file_name)
            if not Over_write:
                Bucket.check(remote_tiles)
            
            Bucket.upload(
                remote_path=remote_tiles,
                local_path=tile
            )

        print("# ===== upload target to bucket.....")

        for target in tqdm(Vec.labellist):
            file_name=target.split('/')[-1]
            remote_target=os.path.join(bucket_targets_root,file_name)
            if not Over_write:
                Bucket.check(remote_target)
        
            Bucket.upload(
                remote_path=remote_target,
                local_path=target
            )
        print("# ===== uploaded bucket:")
        Bucket.ls()

    if not Keep_local:
        print("# ------------------------------- Clear-cache ------------------------------- #")
        cmd="rm -rf "+DataSetName
        os.system(cmd)
        print("# -------------------------------- Clear-Done ------------------------------- #")

    print("# ---------------------------------------------------------------------------- #")
    print("#                             DataSet process done                             #")
    print("# ---------------------------------------------------------------------------- #")

def main():
    vecfile="/workspace/data/osm-2017-07-03-v3.6.1-china_beijing.mbtiles"
    macfile='/Users/tanwenxuan/workspace/Data/osm-2017-07-03-v3.6.1-china_beijing.mbtiles'
    tqfile="/workspace/osm-2017-07-03-v3.6.1-china_beijing.mbtiles"

    process(tqfile,Keep_local=False,Over_write=True)





if __name__ == '__main__':
    main()
    


import sys
from .raster import Raster
import os
import numpy as np
from tqdm import tqdm
import json
import multiprocessing

from osgeo import gdal
from osgeo import ogr
from osgeo import osr
from osgeo import gdal_array
from osgeo import gdalconst

class Vector(Raster):
    def __init__(self, input_shp_path=''):

        print("# ---------------------------------------------------------------------------- #")
        print("#                                Vector Toolkit                                #")
        print("# ---------------------------------------------------------------------------- #")
        Raster.__init__(self)
        self.readshp(input_shp_path)
        self.ExportLayer = None

    def readshp(self, input_shp_path):
        if os.path.isfile(input_shp_path):
            filetype = input_shp_path.split(".")[-1]
            gdal.SetConfigOption("GDAL_FILENAME_IS_UTF8", "NO")
            gdal.SetConfigOption("SHAPE_ENCODING", "")
            ogr.RegisterAll()
            self.DriveDict = {
                'shp': ogr.GetDriverByName('ESRI Shapefile'),
                'SHP': ogr.GetDriverByName('ESRI Shapefile'),
                'Geojson': ogr.GetDriverByName('GeoJSON'),
                'geojson': ogr.GetDriverByName('GeoJSON'),
                'pbf': ogr.GetDriverByName('MVT'),
                'mbtiles': ogr.GetDriverByName('MBTiles')
            }
            if filetype in self.DriveDict.keys():
                print("\033[0m# -----Valid vector format :\033[1;32m ", filetype)
                self.Driver = self.DriveDict[filetype]

            self.Input_path = input_shp_path
            self.DataSource = self.Driver.Open(self.Input_path)
            self.meta = self.DataSource.GetMetadata()
            
            print("\n\033[0m# ----------------------------- Meta Information ----------------------------- #")
            self.print_dict(self.meta)

            print("# ----------------------------- Meta Information ----------------------------- #\n")
            self.Description = self.DataSource.GetDescription()
            print('\033[0m# -----Description :\033[1;32m ', self.Description)
            assert self.DataSource is not None, '\n\n\nERROR-----' + \
                str(input_shp_path) + '  --- Invalid Input Shepefile\n\n\n'
            self.LayerCount = self.DataSource.GetLayerCount()
            print("\033[0m# -----LayerCount\033[1;32m :", self.LayerCount)
            self.LayerDict = {}
            for i in range(self.LayerCount):
                self.Layer = self.DataSource.GetLayer(i)
                print(
                    "\033[0m# -----Layer :\033[1;34m ",
                    i,
                    "\033[0m LayerName : \033[1;32m ",
                    self.Layer.GetName(),
                    self.Layer.GetGeometryColumn())
                self.LayerDict[self.Layer.GetName()] = self.Layer
            
            
            self.Srs = self.Layer.GetSpatialRef()
            self.Extent=self.Layer.GetExtent()
            print("\033[0m# -----Extent:\033[1;32m ",self.Extent)
            print("\033[0m# -----Alread Load:\033[1;32m ", input_shp_path,"\033[0m")
            print(
                "# -------------------------------- DEFINE DONE ------------------------------- #")

        else:
            print("# ----- \033[5;31m Warning: Vector Class init with wrong path or invalid vector file \nPath:",input_shp_path,"\033[0m\n")

    def getDefaultLayerbyName(self, name):
        """
        para:name string of layer name
        """
        self.defaultlayer = self.LayerDict[name]
        print("# -----Set Default Layer \033[1;32m|",name,"| : \033[0m",self.defaultlayer)
        return self.LayerDict[name]

    def Info(self):
        print(self.DataSource)
        print(self.Layer)
        print(self.Srs)

    def getbounding(self):
        adfGeoTransform = self.geotransform
        x1 = adfGeoTransform[0] + 0 * \
            adfGeoTransform[1] + 0 * adfGeoTransform[2]
        y1 = adfGeoTransform[3] + 0 * \
            adfGeoTransform[4] + 0 * adfGeoTransform[5]
        x2 = adfGeoTransform[0] + self.width * \
            adfGeoTransform[1] + self.height * adfGeoTransform[2]
        y2 = adfGeoTransform[3] + self.width * \
            adfGeoTransform[4] + self.height * adfGeoTransform[5]
        self.bounding = (x1, y1, x2, y2)
        return self.bounding

    def getdataset(self, tif_path):
        self.tif_path = tif_path
        self.dataset = gdal.Open(tif_path)
        print('\n# -----Dataset:', self.dataset)
        return self.dataset

    def SaveTo(self, name):
        filetype = name.split(".")[-1]
        if filetype in self.DriveDict.keys():
            out = self.DriveDict[filetype].CopyDataSource(
                self.DataSource, name)
            out = None

    def SaveVectorByLayerName(self, LayerName, outputname, format="GeoJSON"):
        """
        format:
            * ESRI Shapefile
            * GeoJSON
        """
        self.ExportLayer = self.LayerDict[LayerName]
        print("# ------------------Start Create Layer",
              LayerName, ' ---------------------- #')
        self.ExportDriver = ogr.GetDriverByName(format)

        if os.path.isfile(outputname):
            self.ExportDriver.DeleteDataSource(outputname)

        ExportResources = self.ExportDriver.CreateDataSource(outputname)
        print("-----ExportResources:", ExportResources)
        assert not ExportResources is None, "Invalid Export Resources ,Please check the path of output"
        ExportLayerTemp = ExportResources.CreateLayer(
            LayerName, geom_type=ogr.wkbPolygon)
        print("-----Data Resources Create Done")

        # Add input Layer Fields to the output Layer
        inLayerDefn = self.ExportLayer.GetLayerDefn()
        for i in range(0, inLayerDefn.GetFieldCount()):
            fieldDefn = inLayerDefn.GetFieldDefn(i)
            ExportLayerTemp.CreateField(fieldDefn)

        # Get the output Layer's Feature Definition
        outLayerDefn = ExportLayerTemp.GetLayerDefn()

        # Add features to the ouput Layer
        print("-----Start Loop Process")
        for i in tqdm(range(0, self.ExportLayer.GetFeatureCount())):
            # Get the input Feature
            inFeature = self.ExportLayer.GetFeature(i)
            # Create output Feature
            outFeature = ogr.Feature(outLayerDefn)
            # Add field values from input Layer
            for i in range(0, outLayerDefn.GetFieldCount()):
                outFeature.SetField(
                    outLayerDefn.GetFieldDefn(i).GetNameRef(),
                    inFeature.GetField(i))
            # Set geometry as centroid
            geom = inFeature.GetGeometryRef()
            inFeature = None
            centroid = geom.Centroid()
            outFeature.SetGeometry(centroid)
            # Add new feature to output Layer
            ExportLayerTemp.CreateFeature(outFeature)
            outFeature = None

        print("# -----LayerCopyDone,", LayerName)
        ExportResources.SetMetadata(self.meta)
        print("# -----Set Meta Done")
        ExportResources.SetDescription(self.Description)
        print("# -----Set Description Done")
        ExportResources.Destroy()
        print("# --------------------------------- Save Done -------------------------------- #")

        # Save and close DataSources
        inDataSource = None
        outDataSource = None

    def crop_layer_by_polygon(self, vector):
        self.ExportLayer.SetSpatialFilter()

    def crop_layer_by_rect(self, rect):
        assert self.ExportLayer is None, 'Invalid Export Layer'
        self.ExportLayer.SetSpatialFilterRect()

    
    def crop_default_layer_by_rect(self, rect):
        print("# -----Set filter Rect:\033[1;32m",rect,'\033[0m')
        self.defaultlayer.SetSpatialFilterRect(*rect)
    
    def print_dict(self,d,n=0):
        length=67
        for k,v in d.items():
            # print ('\t'*n)
            if type(v)==type({}):
                print("%s : {" % k)
                self.print_dict(v,n+1)
            else:
                
                strl=len(str(k))+len(str(v))
                space=length-strl
                if strl>length:                    
                    v=str(v)[:space]
                print("\033[0m# -----\033[1;34m %s :\033[1;32m  %s\033[0m" % (k,v)+" "*space+"#")
        if n!=0:
            print('\t'*(n-1)+ '}')
    
    def Rasterize(self, outputname, Nodata=0):

        targetDataSet = gdal.GetDriverByName('GTiff').Create(
            outputname, self.width, self.height, 1, gdal.GDT_Byte)
        targetDataSet.SetGeoTransform(self.geotransform)
        targetDataSet.SetProjection(self.projection)
        band = targetDataSet.GetRasterBand(1)
        band.SetNoDataValue(Nodata)
        band.FlushCache()
        gdal.RasterizeLayer(
            targetDataSet,
            [1],
            self.defaultlayer,
            )
        targetDataSet = None
        
    def generate(self,tiles,output_path="./label"):
        print('# ===== \033[5;33mStart Generate.....\033[0m')
        """
        修订  把Rasterize同样使用线程分解的可能性
        """

        self.labellist=[]
        if not os.path.exists(output_path):
            os.makedirs(output_path)
        for tile in tqdm(tiles):
            self.readtif(tile)
            filename=tile.split('/')[-1]
            path=os.path.join(output_path,filename)
            self.labellist.append(path)
            self.Rasterize(path)
        return self.labellist
    
    def reset_layer(self):
        self.Layer.ResetReading()

    # def Json2Voclike(self, out_dir='./VocDataset'):
    #     """

    #     :param label_file:
    #     label.txt & the content like:
    #     __ignore__
    #     _background_
    #     Iron
    #     Mine
    #     None
    #     land

    #     The Content '__ignore__' maybe should add in file by manual
    #     :param in_dir:
    #     Dir input Data like :
    #     in_dir-
    #         |
    #         1.jpg
    #         1.json
    #         2.jpg
    #         2.json
    #         ....
    #     :param out_dir:
    #     output Voclike Dataset Path
    #     :return:
    #     """
    #     print('**********************strart voc-like  process******************')
    #     if os.path.exists(out_dir):
    #         os.removedirs(out_dir)
    #     os.makedirs(out_dir)
    #     os.makedirs(os.path.join(out_dir, 'JPEGImages'))
    #     os.makedirs(os.path.join(out_dir, 'SegmentationClass'))
    #     os.makedirs(os.path.join(out_dir, 'SegmentationClassPNG'))
    #     os.makedirs(os.path.join(out_dir, 'SegmentationClassVisualization'))

    #     class_names = []
    #     class_name_to_id = {}
    #     # for i, line in enumerate(open(labels_file).readlines()):
    #     #     class_id = i - 1  # starts with -1
    #     #     class_name = line.strip()
    #     #     print('current class name :', class_name)
    #     #     class_name_to_id[class_name] = class_id
    #     #     if class_id == -1:
    #     #         assert class_name == '__ignore__'
    #     #         continue
    #     #     elif class_id == 0:
    #     #         assert class_name == '_background_'
    #     #     class_names.append(class_name)
    #     assert self.TYPE is not None, 'None type list Please Set First'
    #     class_names = tuple(self.TYPE)
    #     self.classlist = class_names
    #     self.outputdir = out_dir
    #     print('class_names:', class_names)

    #     out_class_names_file = os.path.join(out_dir, 'class_names.txt')
    #     with open(out_class_names_file, 'w') as f:
    #         f.writelines('\n'.join(class_names))
    #     print('Saved class_names:', out_class_names_file)

    #     colormap = labelme.utils.label_colormap(255)
    #     in_dir = self.LabelmeJsonPath
    #     for label_file in glob.glob(os.path.join(in_dir, '*.json')):
    #         print('Generating dataset from:', label_file)
    #         with open(label_file) as f:
    #             base = os.path.splitext(os.path.basename(label_file))[0]
    #             out_img_file = os.path.join(
    #                 out_dir, 'JPEGImages', base + '.jpg')
    #             out_lbl_file = os.path.join(
    #                 out_dir, 'SegmentationClass', base + '.npy')
    #             out_png_file = os.path.join(
    #                 out_dir, 'SegmentationClassPNG', base + '.png')
    #             out_viz_file = os.path.join(
    #                 out_dir, 'SegmentationClassVisualization', base + '.jpg')

    #             data = json.load(f)

    #             img_file = os.path.join(os.path.dirname(label_file), data['imagePath'])
    #             img = np.asarray(PIL.Image.open(img_file))
    #             PIL.Image.fromarray(img).save(out_img_file)

    #             lbl = labelme.utils.shapes_to_label(
    #                 img_shape=img.shape,
    #                 shapes=data['shapes'],
    #                 label_name_to_value=class_name_to_id,
    #             )
    #             labelme.utils.lblsave(out_png_file, lbl)

    #             np.save(out_lbl_file, lbl)

    #             viz = labelme.utils.draw_label(
    #                 lbl, img, class_names, colormap=colormap)
    #             PIL.Image.fromarray(viz).save(out_viz_file)

   # def Shp2LabelmeJson(self, output_GeoJson=True,
    #                     Labelme_Json_path='./Segmentation/',
    #                     invert2Voc=True, FieldClassName='TYPE'):
    #     """
    #     invert the Shapefile to labelme_Json
    #     Flow I  : Read Tif & Shapefile -> Dataset & DataSource
    #     Flow II : Init Null labelme_json dict
    #     Flow III: Get Cords(x,y) & Feature and invert longtitude,latitude to image cols,rows
    #     Flow IV : Append image Cords list to dict
    #     Flow V  : Output Labelme Json
    #     Flow VI : Use System Command labelme_json_to_dataset / labelme2voc.py
    #             # It generates:
    #             #   - data_dataset_voc/JPEGImages
    #             #   - data_dataset_voc/SegmentationClass
    #             #   - data_dataset_voc/SegmentationClassVisualization
    #     :param shp_path:
    #     :param tif_path:
    #     :param Labelme_Json_path: Output Json&jpg path
    #     :param invert2Dataset: invert labelme json to Dataset
    #     :param  TYPE like (['Storge yard', 'Container', 'Oil Tank', 'Berth'])
    #     :return:
    #     """

    #     print('-----feature count :', self.Layer.GetFeatureCount())
    #     print("-----SpatialFilter :", self.Layer.GetSpatialFilter())

    #     print("-----", self.Layer.GetLayerDefn())

    #     print("-----", self.Layer.GetFeatureCount())

    #     print("-----", self.Layer.GetExtent())

    #     x1, y1, x2, y2 = self.Layer.GetExtent()
    #     x1, y1 = self.geo2imagexy(x1, y1)
    #     x2, y2 = self.geo2imagexy(x2, y2)
    #     print("-----Image:", x1, y1, x2, y2)
    #     width = abs(x2 - x1)
    #     height = abs(y2 - y1)
    #     print("-----Whole Image shape:", height, width)

    #     print('-----', self.Layer[0])

    #     print("-----", self.Layer.GetSpatialRef())
    #     exit(0)

    #     # for i in tqdm(range(self.Layer.GetFeatureCount())):
    #     #     in_Feature = self.Layer.GetFeature(i)
    #     #     geom = in_Feature.GetGeometryRef()
    #     #     print(type(geom))
    #     #     print(geom)
    #     # FieldValue=in_Feature.GetField(FieldClassName)
    #     # geodict=geom.ExportToJson()
    #     # geodict=

    #     # geodict=json.loads(geodict)

    #     # feature=self.initcordarrary(self.TYPE[FieldValue],imagecord)

    #     # shapes.append(feature)
    #     # print('\n\nshape is :',shapes,' process done \n\n')
    #     # return 0
    #     ####################################################################
    #     # shapes process done
    #     if(not os.path.exists(Labelme_Json_path)):
    #         os.system('mkdir ' + Labelme_Json_path)
    #     filename = filename.split('.')[-2]
    #     self.LabelmeJsonPath = Labelme_Json_path
    #     outputjsonname = Labelme_Json_path + filename + '.json'
    #     self.save(outputjsonname, shapes, filename + '.jpg', imageData=None,
    #               lineColor=(255, 0, 0, 128), fillColor=(0, 255, 0, 128))

    #     Newim = Image.fromarray(self.tif.image_nparray)
    #     Newim.save(Labelme_Json_path + filename + '.jpg')
    #     print('\n\n****json   &   image save done *****\n\n')

    #     if output_GeoJson:
    #         geojsonfilename = self.Input_path + '.geojson'
    #         formattransfrom_command = 'ogr2ogr -f GeoJSON ' + \
    #             geojsonfilename + ' ' + self.Input_path
    #         print(formattransfrom_command)
    #         os.system(formattransfrom_command)
    #         print(' output geojson transfrom done ')

    #     if invert2Voc:
    #         self.Json2Voclike()

    #         # command='labelme_json_to_dataset '+outputjsonname
    #         # os.system(command)
    #     #
    #     # if invertPascalVoc:
    #     #     imageDataPIL.save(Labelme_Json_path+filename+'.jpg')

 
def main():
    Vector = SHP("/workspace/data/Water/Beijing.geojson")
    # Vector=SHP("/workspace/SQCV/Data/IO/waterchina.geojson")

    # ------------------------------ Rasterize Demo ------------------------------ #

    # Vector=SHP("/home/winshare/Downloads/2017-07-03_asia_china.mbtiles").SaveTo("CHINA.shp")
    # Vector.SaveVectorByLayerName('water',"waterchina.shp",format="ESRI Shapefile")

    import glob

    tiflist = glob.glob("/workspace/data/Water/GoogleBeijing/*.tif")
    # tiflist.extend(glob.glob("/workspace/data/Water/BingBeijing/*.tif"))

    # Experiment--Geojson
    # ----------------------------------- Speed ---------------------------------- #
    #                               十万样本时间
    # CHINA             34          94  小时
    # Beijing           1/13        120 分钟
    # MBtilesFilter     3           8.3 小时

    # Experiment--Shapefile
    # ----------------------------------- Speed ---------------------------------- #

    print(len(tiflist))
    for file in tqdm(tiflist):
        Vector.readtif(file)
        Vector.getDefaultLayerbyName("bui")

        filename = file.split('/')[-1]
        path = '/workspace/data/Water/label/' + filename
        Vector.Rasterize(path)


if __name__ == '__main__':
    main()



# **************************************************************************** #
#                                                                              #
#                                                         :::      ::::::::    #
#    raster.py                                          :+:      :+:    :+:    #
#                                                     +:+ +:+         +:+      #
#    By: Tom Winshare <tanwenxuan@live.com>         +#+  +:+       +#+         #
#                                                 +#+#+#+#+#+   +#+            #
#    Created: 2020/04/20 17:05:30 by winshare          #+#    #+#              #
#    Updated: 2020/09/16 18:15:16 by Tom Winshar      ###   ########.fr        #
#                                                                              #
# **************************************************************************** #

# Copyright 2020 winshare
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys
import os
from osgeo import gdal
from osgeo import osr
from osgeo import ogr
import gdal
import glob
import numpy as np
import PIL.Image as Image


class Raster():
    def __init__(self, filename=None, channel=[
                 0, 1, 2], debug=False):
        """
        filename: could be filedir or path of single file
        """
        print("# ---------------------------------------------------------------------------- #")
        print("#                            TIFF process Toolkit                              #")
        print("# ---------------------------------------------------------------------------- #")
        # self.display = display
        self.debug = debug
        self.channel = channel
        if not filename is None:
            if os.path.isfile(filename):
                print("# -----TIFF Class Init with :", filename)
                self.filename = filename
                self.readtif(self.filename)
        else:
            print("# -----Class TIF init without filename")

    # ---------------------------------------------------------------------------- #
    #                                     Init                                     #
    # ---------------------------------------------------------------------------- #

    def readtif(self, filename):
        self.dataset = gdal.Open(filename)  # 打开文件
        assert self.dataset is not None, "Can't Read Dataset ,Invalid tif file : " + filename
        self.width = self.dataset.RasterXSize  # 栅格矩阵的列数
        self.height = self.dataset.RasterYSize  # 栅格矩阵的行数
        self.geotransform = self.dataset.GetGeoTransform()  # 仿射矩阵
        self.projection = self.dataset.GetProjection()  # 地图投影信息
        self.image = self.dataset.ReadAsArray(0, 0, self.width, self.height)
        # print('-----Original Data Shape : ',self.image.shape)
        if 'uint8' in self.image.dtype.name:
            self.datatype = gdal.GDT_Byte
            # print('image type : uint8')
        elif 'int8' in self.image.dtype.name:
            # print('image type : int8')
            self.datatype = gdal.GDT_Byte
        elif 'int16' in self.image.dtype.name:
            # print('image type : int16')
            self.datatype = gdal.GDT_UInt16
        else:
            # print('image type : float32')
            self.datatype = gdal.GDT_Float32
        if len(self.image.shape) == 2:
            self.channel_count = 1
        if len(self.image.shape) == 3:
            self.channel_count, _, _ = self.image.shape
            if self.channel_count > 20:
                _, _, self.channel_count = self.image.shape
            else:
                self.image = self.image.transpose(1, 2, 0)
                self.image = self.image[:, :, self.channel[:]]


    # def displayimagery(self):
    #     self.percentage = self.fast_percentager_strentching(self.image)
    #     plt.imshow(self.percentage), plt.show()

    # ---------------------------------------------------------------------------- #
    #                                     Read                                     #
    # ---------------------------------------------------------------------------- #

    # ---------------------------------------------------------------------------- #
    #                         fast_percentager_strentching                         #
    # ---------------------------------------------------------------------------- #

    def fast_percentager_strentching(
            self, image=None, percentage=2, sample=10000):
        """
        Image ndarray:(W,H,C)
        Percentage N(0-100)%
        """

        assert not percentage > 100 or percentage < 0, "Invalde Percentage Value"
        print(
            "# -------------------------- percentager_strentching -------------------------")
        print(
            "# ------------------- process with percentage : ",
            percentage,
            "% ------------------")
        percentage = percentage / 100
        if isinstance(image, None):
            image = self.image
        W, H = image.shape[0], image.shape[1]
        w = np.random.randint(0, W, sample)
        h = np.random.randint(0, H, sample)
        if len(image.shape) == 3:
            points = image[w, h, :]
            point = [np.mean(channels) for channels in points]
        else:
            points = image[w, h]
            point = points
        pointset = sorted(point)
        min = int(sample * percentage)
        max = int(sample * (1 - percentage))
        min = pointset[min]
        max = pointset[max]
        image[image > max] = max
        image[image < min] = min
        image = (image - min) / (max - min)
        print("# ----- Max : ", max, " Min :    ", min, "-----")
        self.image = image
        return image

    def setdata(self, Data):
        """
        write a new raster file with bool type data
        """
        if len(Data.shape) == 2:
            assert not isinstance(
                Data[0, 0], bool), 'Polygonize Data ( SetRasterData(Data) ) Must be bool Ndarray,But now in ' + str(type(Data[0, 0]))
        self.RasterSet = True
        self.imageoutput = Data

    def writeimagery(self, name=None, format="png"):
        if name is None:
            name = self.filename + "_imagery."+format
        image=Image.fromarray(self.image)
        # cv2.imwrite(name, self.image)
        image.save(name)

    def writetif(self, outputname,):
        """
        write file in tiff format
        """
        pass

    def resize_raster(self, resize_ratio=0.5):
        """
        cv2 resize image data
        6 parameter 1,5 is resolution ratio  its need /resize_ratio
        """
        size = (int(self.width * resize_ratio),int(self.height * resize_ratio))
        image=Image.fromarray(self.image_nparray)
        image.resize(size,resample=PIL.Image.BILINEAR)
        # self.resizedimage = cv2.resize(
        #     self.image_nparray, size, interpolation=cv2.INTER_AREA)
        self.resizedimage=np.array(image)
        self.ResizeGeo = list(self.geotrans)
        print('input Geo parameter, :', self.ResizeGeo)
        self.ResizeGeo[1] = float(self.ResizeGeo[1] / resize_ratio)
        self.ResizeGeo[5] = float(self.ResizeGeo[5] / resize_ratio)
        print('resized Geo parameter ，：', self.ResizeGeo)
        self.geotrans = tuple(self.ResizeGeo)
    # ready for fix the resize 
    def writethreshold2shp(self):
        """
        Set the Boolmap(ndarray) & do polygonize in boolmap to save
        :return:
        """
        assert self.dataset is not None, 'Null dataset'
        assert self.RasterSet, 'Please Set Bool map in ndarray with SetRasterData() \n, Current output polygon src band is ' + \
            str(self.imageoutput)
        shp_name = self.out_middle_tif_name + '_polygonized.shp'
        srcband = self.dataset.GetRasterBand(1)
        maskband = None
        format = 'ESRI Shapefile'
        drv = ogr.GetDriverByName(format)
        dst_ds = drv.CreateDataSource(shp_name)
        srs = osr.SpatialReference()
        srs.ImportFromWkt(self.outdataset.GetProjectionRef())

        dst_layer = dst_ds.CreateLayer(
            shp_name, geom_type=ogr.wkbPolygon, srs=srs)
        if (dst_layer is None):
            return 0, 0
        dst_field = dst_layer.GetLayerDefn().GetFieldIndex(shp_name)
        prog_func = gdal.TermProgress
        options = []
        result = gdal.Polygonize(srcband, maskband, dst_layer, dst_field, options,
                                 callback=prog_func)
        dst_ds = None
        print('Shapefile has write in ', shp_name)
        return shp_name

    def clear(self):
        print('-----TIF Object has been init with null')
        self.geotransform = None
        self.image = None
        self.dataset = None
        self.projection = None

    def createdataset(self, out_put_tif_name):
        driver = gdal.GetDriverByName("GTiff")  # 数据类型必须有，因为要计算需要多大内存空间
        self.outputdataset = driver.Create(
            out_put_tif_name,
            self.width,
            self.height,
            self.channel_count,
            self.datatype)
        self.outputdataset.SetGeoTransform(self.geotransform)  # 写入仿射变换参数
        self.outputdataset.SetProjection(self.projection)  # 写入投影
        print('Create Dataset With ', self.dataset)
        print('Create Shape is ', (self.height, self.width))

    # --------------------------------- Transform -------------------------------- #
    # ---------------------------------------------------------------------------- #
    #                                Cord Transform                                #
    # ---------------------------------------------------------------------------- #

    def getSRSPair(self):
        '''
        获得给定数据的投影参考系和地理参考系
        :param dataset: GDAL地理数据
        :return: 投影参考系和地理参考系
        '''
        prosrs = osr.SpatialReference()
        prosrs.ImportFromWkt(self.dataset.GetProjection())
        geosrs = prosrs.CloneGeogCS()
        return prosrs, geosrs

    def geo2lonlat(self, x, y):
        '''
        将投影坐标转为经纬度坐标（具体的投影坐标系由给定数据确定）
        :param dataset: GDAL地理数据
        :param x: 投影坐标x
        :param y: 投影坐标y
        :return: 投影坐标(x, y)对应的经纬度坐标(lon, lat)
        '''
        prosrs, geosrs = self.getSRSPair()
        ct = osr.CoordinateTransformation(prosrs, geosrs)
        coords = ct.TransformPoint(x, y)
        return coords[:2]

    def lonlat2geo(self, lon, lat):
        '''
        将经纬度坐标转为投影坐标（具体的投影坐标系由给定数据确定）
        :param dataset: GDAL地理数据
        :param lon: 地理坐标lon经度
        :param lat: 地理坐标lat纬度
        :return: 经纬度坐标(lon, lat)对应的投影坐标
        '''
        prosrs, geosrs = self.getSRSPair()
        ct = osr.CoordinateTransformation(geosrs, prosrs)
        coords = ct.TransformPoint(lon, lat)
        
        return coords[:2]

    def imagexy2geo(self, row, col):
        '''
        根据GDAL的六参数模型将影像图上坐标（行列号）转为投影坐标或地理坐标（根据具体数据的坐标系统转换）
        :param dataset: GDAL地理数据
        :param row: 像素的行号
        :param col: 像素的列号
        :return: 行列号(row, col)对应的投影坐标或地理坐标(x, y)
        '''
        trans = self.dataset.GetGeoTransform()
        px = trans[0] + col * trans[1] + row * trans[2]
        py = trans[3] + col * trans[4] + row * trans[5]
        return px, py

    def geo2imagexy(self, x, y):
        '''
        根据GDAL的六 参数模型将给定的投影或地理坐标转为影像图上坐标（行列号）
        :param dataset: GDAL地理数据
        :param x: 投影或地理坐标x
        :param y: 投影或地理坐标y
        :return: 影坐标或地理坐标(x, y)对应的影像图上行列号(row, col)
        '''
        trans = self.dataset.GetGeoTransform()
        a = np.array([[trans[1], trans[2]], [trans[4], trans[5]]])
        b = np.array([x - trans[0], y - trans[3]])
        return np.linalg.solve(a, b)  # numpy linalg.solve equation

    def lonlat2imagexy(self, x, y):
        x1, y1 = self.lonlat2geo(x, y)
        x2, y2 = self.geo2imagexy(x1, y1)
        return x2, y2

    def imagexy2lonlat(self, x, y):
        x1, y1 = self.imagexy2geo(x, y)
        x2, y2 = self.geo2lonlat(x1, y1)
        return x2, y2

    def getfiles_from_dir(self, dir):
        """
        return the filename list of tiff file from dir
        """
        assert not os.path.isdir(dir), "Invalid dir format" + str(dir)
        print("# -----Read Dir :", dir)
        self.files = glob.glob(os.path.join(dir, "./*.tif"))


def main():
    """
    This part will show the standard function guide.
    """


if __name__ == '__main__':
    main()



from obs import ObsClient



class bucket():
    def __init__(self,
    access_key_id=None,
    secret_access_key=None,
    server=None,
    bucketName=None
    ):
    # 创建ObsClient实例
        self.base_folder="/"
        if access_key_id==None or secret_access_key==None or server== None:
            print("# ----- Error invalid OBS server config please check parameter of obs server")
            exit(0)
        else:
            self.obsClient = ObsClient(
            access_key_id=access_key_id,    
            secret_access_key=secret_access_key,    
            server=server
            )

        if bucketName!=None:
            self.bucketName=bucketName
            self.bucketClient=self.obsClient.bucketClient(bucketName)

        print("# ---------------------------------------------------------------------------- #")
        print("#                                Bucket ToolKit                                #")
        print("# ---------------------------------------------------------------------------- #")       
        print("# ----access key (AK) : ",access_key_id)
        print("# ----secret key (SK): ",secret_access_key)
        print("# ----server : ",server)
        print("# ----bucket name : ",bucketName)
        print("# ----root : ",self.base_folder)
        print("# ---------------------------------------------------------------------------- #")       
        


    def getBucketMetadata(self):
        print('Getting bucket metadata\n')
        #resp = obsClient.getBucketMetadata(bucketName, origin='http://www.b.com', requestHeaders='Authorization1')
        resp = self.bucketClient.getBucketMetadata(origin='http://www.b.com', requestHeaders='Authorization1')
        print('storageClass:', resp.body.storageClass)
        print('accessContorlAllowOrigin:', resp.body.accessContorlAllowOrigin)
        print('accessContorlMaxAge:', resp.body.accessContorlMaxAge)
        print('accessContorlExposeHeaders:', resp.body.accessContorlExposeHeaders)
        print('accessContorlAllowMethods:', resp.body.accessContorlAllowMethods)
        print('accessContorlAllowHeaders:', resp.body.accessContorlAllowHeaders)
        print('Deleting bucket CORS\n')
        resp = self.bucketClient.deleteBucketCors()
        print('status'  + str(resp.status))
        return resp

    def upload(self,remote_path,local_path):
        self.obsClient.putFile(self.bucketName,remote_path, local_path)
        # print("# ===== Uploading ",local_path," ===to : ",remote_path)

    
    def download(self,key,download):
        # print("# ===== Downloading ",key," === to :",download)
        self.obsClient.getObject(self.bucketName, key, downloadPath=download)


    def cd(self,folder_key):
        self.base_folder=folder_key
        print("# ===== Base Folder",self.base_folder)


    def delete(self,key):
        print('# ===== Deleting object ' +key + '\n')
        self.obsClient.deleteObject(self.bucketName, key)

    def check(self,key):
        """
        The Sync will overwrite by default. We need check
        """
        assert not self.obsClient.getObject(self.bucketName,key)["status"]<300,"\n# ===== ERROR : \n# ===== bucket : ({bucketname})\n# ===== key : ({key}) & local upload flow try to overwrite same key".format(bucketname=self.bucketName,key=key)


    # def mkdir(self,dir):
    #     pass


    def ls(self,show_item_count=10):
        print("# ===== list ({path}): ".format(path=self.base_folder))
        # resp = self.obsClient.listObjects(self.bucketName)

        if self.base_folder=="/":
            resp = self.obsClient.listObjects(self.bucketName)
            
        else:
            resp = self.obsClient.listObjects(self.bucketName,self.base_folder)
        keylist=[]
        print("# ===== object count : ",len(resp.body.contents))
        for content in resp.body.contents[:show_item_count]:
            keylist.append(content.key)
            print('   |--- : ' + content.key + ' etag[' + content.etag + ']')
        return keylist

def main():
    Bucket=bucket()
    # Bucket.upload("/port.vin","./Data/IO/port.vin")
    # Bucket.download("/port.vin","./hash.vin")
    # Bucket.getBucketMetadata()
    Bucket.cd("Description/")
    Bucket.ls()
    Bucket.delete("Description/")




if __name__ == '__main__':
    main()
    


# Copyright 2020 winshare
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()




setuptools.setup(
    name="d-arth", # Replace with your own username
    version="0.1.3",
    author="Winshare Tom",
    author_email="tanwenxuan@live.com",
    description="The Satellite Imagery DataSet Toolkit",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/OOXXXXOO/DARTH",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "certifi==2020.6.20",
        "decorating==0.6.1",
        "esdk-obs-python==3.20.1",
        "tqdm==4.46.1"
        "olefile==0.46",
        "python-slugify==3.0.3",
        "text-unidecode==1.2",
        "GDAL",
        "numpy",
        "Pillow",
    ],
    license='Apache License Version 2.0',
    python_requires='>=3.6',
)



# Copyright 2020 winshare
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# **************************************************************************** #
#                                                                              #
#                                                         :::      ::::::::    #
#    superdownloader.py                                 :+:      :+:    :+:    #
#                                                     +:+ +:+         +:+      #
#    By: winshare <tanwenxuan@live.com>             +#+  +:+       +#+         #
#                                                 +#+#+#+#+#+   +#+            #
#    Created: 2020/05/25 15:10:08 by winshare          #+#    #+#              #
#    Updated: 2020/05/25 15:10:08 by winshare         ###   ########.fr        #
#                                                                              #
# **************************************************************************** #

# ---------------------------------- STD Lib --------------------------------- #

from threading import Thread
import multiprocessing
import time
import os
import io
import PIL.Image as Image
import numpy as np
from tqdm import tqdm

# -------------------------------- Custom Lib -------------------------------- #
import urllib.request as ur
from .subscription import MAP_URLS,Tilesize,AGENT
from .transform import getExtent,wgs_to_tile,TileXYToQuadKey,saveTiff,wgs_to_mercator


title="""

# ---------------------------------------------------------------------------- #
# ----------------- \033[5;31m██████╗  █████╗ ██████╗ ████████╗██╗  ██╗ \033[0m---------------- #
# ----------------- \033[5;32m██╔══██╗██╔══██╗██╔══██╗╚══██╔══╝██║  ██║ \033[0m---------------- #
# ----------------- \033[5;33m██║  ██║███████║██████╔╝   ██║   ███████║ \033[0m---------------- #
# ----------------- \033[5;34m██║  ██║██╔══██║██╔══██╗   ██║   ██╔══██║ \033[0m---------------- #
# ----------------- \033[5;35m██████╔╝██║  ██║██║  ██║   ██║   ██║  ██║ \033[0m---------------- #
# ----------------- \033[5;35m╚═════╝ ╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚═╝  ╚═╝ \033[0m---------------- #
# ---------------------------------------------------------------------------- #
"""

class downloader(Thread):
    def __init__(self, server=None,thread_count=4,format='tif'):


        print(title)
        print("# ---------------------------------------------------------------------------- #")
        print("#                            MAP Production Toolkit                            #")
        print("# ---------------------------------------------------------------------------- #")
        # print("Support DataSources",MAP_URLS.keys())
        self.thread_count=thread_count
        self.format=format
        if server is not None and server in MAP_URLS.keys():

            print(
                "# ---------------------- MAP Serverv Init Successful by ---------------------- #")
            lines = 52
            length = len(server)
            space = lines - length
            print("# ----------------------\033[5;32m", server, space * "\033[0m-", "#")
            self.server = server
        else:
            self.server = "Google"



    def add_cord(self, left, top, right, bottom, zoom, style='s'):
        """
        Compute the Extent by (x1,y1,x2,y2)-Retangle coordinate A
        I: coordinate A->WGS Extent of tile
        II: Extent->Tile coordinate 
        III:Tile coordinate-> array of x,y coordinates WGS left top point in each tile
        """

        # ------------------------------------- I ------------------------------------ #

        self.left, self.top, self.right, self.bottom, self.zoom = left, top, right, bottom, zoom
        self.extent = getExtent(
            self.left,
            self.top,
            self.right,
            self.bottom,
            self.zoom,
            self.server)

        # ------------------------------------ II ------------------------------------ #

        self.pos1x,self.pos1y = wgs_to_tile(self.left, self.top, self.zoom)
        pos2x, pos2y = wgs_to_tile(self.right, self.bottom, self.zoom)
        self.lenx = pos2x - self.pos1x + 1
        self.leny = pos2y - self.pos1y + 1
        width = self.lenx * Tilesize
        height = self.leny * Tilesize
        left_top_x,left_top_y=self.extent["LT"][0],self.extent["LT"][1]
        self.resolution_x=(self.extent['RB'][0] - self.extent['LT'][0]) / width
        self.resolution_y=(self.extent['RB'][1] - self.extent['LT'][1]) / height
        right_bottom_x=left_top_x+width*self.resolution_x
        right_bottom_y=left_top_y+height*self.resolution_y
        self.wsg_cord=(left_top_x,left_top_y,right_bottom_x,right_bottom_y)
      
        self.mercator_cord=(*wgs_to_mercator(left_top_x,left_top_y),*wgs_to_mercator(right_bottom_x,right_bottom_y))


        print('# -----WGS BoundingBox: \033[1;32m%s \033[0m'%str(self.wsg_cord))
        print("# -----Mercator BoudingBox: \033[1;32m%s \033[0m"%str(self.mercator_cord))
        # ------------------------------------ III ----------------------------------- #

        self.x_step_point=np.arange(left_top_x,right_bottom_x,self.resolution_x*Tilesize)
        self.y_step_point=np.arange(left_top_y,right_bottom_y,self.resolution_y*Tilesize)
      
        # ----------------------------- URL -> URL Queue ----------------------------- #

        self.urls=self.get_urls(left, top, right, bottom, zoom, style)
        self.urls_queue=multiprocessing.Manager().Queue()
        for data in self.urls:
            self.urls_queue.put(data)

        print("# -----Url Queue size: \033[1;32m%s \033[0m"%self.urls_queue.qsize())


    def downloadurl(self,urls_queue):

        # -------------------------- process the single url -------------------------- #
        t_start = time.time()
        instance=urls_queue.get()
        url=instance["url"]
        HEADERS = {'User-Agent': AGENT}
        header = ur.Request(url, headers=HEADERS)
        err = 0
        while(err < 3):
            try:
                data = ur.urlopen(header).read()
            except BaseException:
                err += 1
            else:
                picio = io.BytesIO(data)
                small_pic = Image.open(picio)
                smallarray = np.array(small_pic)
                
                info=instance["info"]
                x=info[0]
                y=info[1]
                xx=self.x_step_point[x-self.pos1x]
                yy=self.y_step_point[y-self.pos1y]

                Proj=[
                    xx,
                    self.resolution_x,
                    0,
                    yy,
                    0,
                    self.resolution_y
                ]
            
                name="{server}-{x}-{y}-{z}".format(x=info[0],y=info[1],z=info[2],server=self.server)
                path=os.path.join(self.output_path,name)

                # ------------------------------------ I/O ----------------------------------- #

                if self.format=='png':
                    path+='.png'
                    image=Image.fromarray(smallarray)
                    image.save(path)
                    # cv2.imwrite(path,smallarray)
                if self.format=='tif':
                    path+='.tif'
                    if len(smallarray.shape) == 3:
                        h, w, c = smallarray.shape
                        if c == 3:
                            image=Image.fromarray(smallarray)
                            r, g, b = image.split()
                            saveTiff(r, g, b, Proj, path)
                        if c == 4:
                            image=Image.fromarray(smallarray)
                            r, g, b, a = image.split()
                            saveTiff(r, g, b, Proj, path)
                    if len(smallarray.shape) == 2:
                        saveTiff(
                        smallarray,
                        smallarray,
                        smallarray,
                        Proj,
                        path)

                instance["path"]=path
                t_stop = time.time()
         
                return instance

        raise Exception("Bad network link.")
      

    def download(self,output_path="./images"):   
        # print('# -----Queue Downloading with ',self.thread_count,' Process')
        self.output_path=output_path
        if not os.path.exists(self.output_path):
            os.makedirs(self.output_path)
        result_vector=[]
        


        # -------------------------------- Thread Pool ------------------------------- #
        bar=tqdm(total=len(self.urls))
        update=lambda *args:bar.update()
        pool=multiprocessing.Pool(self.thread_count)
        for index,url in enumerate(self.urls):
            result=pool.apply_async(self.downloadurl,args=(self.urls_queue,),callback=update)
            result_vector.append(result)
        
        pool.close()
        pool.join()



        # ------------------------------ process result ------------------------------ #
        print("# ===== Decode Downloading...")
        self.result=[i.get() for i in tqdm(result_vector)]

       
        print("# ------------------------------- Download Done ------------------------------ #")

        name="{server}-{time_}-{rect}-{zoom}.json".format(
            server=self.server,
            time_=str(time.asctime(time.localtime(time.time()))),
            rect=str((self.left,self.top,self.right,self.bottom)),
            zoom=self.zoom
        )
        json_path=os.path.join("".join(output_path.split("/")[:-2]),name)
        self.json_path=json_path
        tileinfo={
            "time":str(time.asctime(time.localtime(time.time()))),
            "left":self.left,
            "top":self.top,
            "right":self.right,
            "bottom":self.bottom,
            "zoom":self.zoom,
            "server":self.server,
            "data":self.result
            }

        # ------------------- save the meta information for result ------------------- #

        import json
        with open(json_path,"w") as jfp:
            json.dump(tileinfo,jfp)
            print("\n# ===== Save description done\033[1;32m",json_path,"\033[0m")

     
    def get_urls(self,x1, y1, x2, y2, z, style):

        # ------------------------------- Get url list ------------------------------- #

        print("# -----Total tiles number：\033[1;32m{x} X {y}\033[0m".format(x=self.lenx, y=self.leny))
        self.urls = [self.get_url(
                i,
                j,
                z,
                style) for j in range(self.pos1y,self.pos1y +self.leny) for i in range(self.pos1x,self.pos1x +self.lenx)
                ]
        return self.urls

    def get_url(self, x, y, z, style):

        # ------------------------------ Resources Check ----------------------------- #

        if self.server == 'Google China':
            url = MAP_URLS["Google China"].format(x=x, y=y, z=z, style=style)
        elif self.server == "Bing VirtualEarth":
            quadkey = TileXYToQuadKey(x, y, z)
            url = MAP_URLS["Bing VirtualEarth"].format(q=quadkey)
        elif self.server == 'Google':
            url = MAP_URLS["Google"].format(x=x, y=y, z=z, style=style)
        else:
            url = MAP_URLS[self.server].format(x=x, y=y, z=z)

        data={}
        data["server"]=self.server
        data["info"]=[x,y,z]
        data["url"]=url
        return data

    def merge(self,path="./result.tif"):

        # ---------------------- Merge tiles to whole tiff file ---------------------- #

        output_array=Image.new("RGBA",(self.lenx*Tilesize,self.leny*Tilesize))
        assert not (self.result==None or len(self.result)==0),"Invalide result of process please check the result of download "+str(self.result)
        for i in self.result:
            x,y,z=i["info"]
            small=Image.open(i['path'])
            x=(x-self.pos1x)*Tilesize
            y=(y-self.pos1y)*Tilesize
            output_array.paste(small,(x,y))

        output_array=output_array.convert("RGB")
        image=Image.fromarray(np.array(output_array))
        r, g, b = image.split()
        gt=(
            self.extent['LT'][0],
            (self.extent['RB'][0] - self.extent['LT'][0]) / r.shape[1],
            0,
            self.extent['LT'][1],
            0,
            (self.extent['RB'][1] - self.extent['LT'][1]) / r.shape[0]
        )
        saveTiff(r,g,b,gt,path)

        print('# ----------------------------- Merge tiles done ----------------------------- #')



def main():
    Google=downloader("Google Satellite")
    Google.add_cord(116.3, 39.9, 116.6, 39.7, 13)#WGS Form
    Google.download()
    # Google.merge()
    # tiles=[i["path"] for i in Google.result]
    # from vector import Vector
    # Building=Vector('/home/winshare/Downloads/2017-07-03_asia_china.mbtiles')
    # Building.getDefaultLayerbyName("building")  
    # Building.crop_default_layer_by_rect(Google.mercator_cord)#FILTER to speed up
    # Building.generate(tiles)




if __name__ == '__main__':
    main()
    